/* SPDX-License-Identifier: LGPL-2.1+ */

#ifndef foodefhfoo
#define foodefhfoo

#define NODES_MAX 64

#define BUFFER_SIZE (64U*1024U)

#if HAVE_SELINUX
#define SUPPORTED_FEATURE_MASK CA_FORMAT_FEATURE_FLAGS_MAX
#else
#define SUPPORTED_FEATURE_MASK (CA_FORMAT_FEATURE_FLAGS_MAX &~ CA_FORMAT_WITH_SELINUX)
#endif

#define SUPPORTED_WITH_MASK (CA_FORMAT_WITH_MASK & SUPPORTED_FEATURE_MASK)

#endif
